import math
from pharmacokinetics import _handlers
from pharmacokinetics import multiple_doses, single_dose

__all__ = [
    "concentrationAtTime",
]

def concentrationAtTime(dose: float, **kwargs) -> float:
    """Calculates the concentration at a specific time (oral administration).

    :param dose: the dosage administered
    :param **kwargs:
        vd: volume of distribution (default 1)
        bioavailability: fraction of drug absorbed into the systemic circulation (default 1)
        ke: elimination rate constant
        t12: elimination half-life (alternative to `ke`)
        ka: absorption rate constant
        t12abs: absorption half-life
        elapsed: time since drug administration
        interval: interval between doses (optional)"""
    ka = _handlers._getPkConstantFromKwargs("ka", "t12abs", kwargs)
    ke = _handlers._getPkConstantFromKwargs("ke", "t12", kwargs)
    bioavailability = kwargs.get("bioavailability", 1)
    elapsed = kwargs.get("elapsed")
    vd = kwargs.get("vd", 1)
    interval = kwargs.get("interval")
    isMultiDose = interval != None

    if isMultiDose:
        fraction = (bioavailability * dose * ka) / (vd * (ka - ke))
        multiplier = math.exp(-ke * elapsed) / (1 - math.exp(-ke * interval))
        multiplier -= math.exp(-ka * elapsed) / (1 - math.exp(-ka * interval))
        return fraction * multiplier
    if ka == ke: # different formula required to prevent undefined value
        k = float(ka)
        fraction = bioavailability * dose * k / vd
        return fraction * elapsed * math.exp(-k * elapsed)
    fraction = (bioavailability * dose * ka) / (vd * (ka - ke))
    multiplier = math.exp(-ke * elapsed) - math.exp(-ka * elapsed)
    return fraction * multiplier



import math

__all__ = [
    "calculateConstant",
    "halflifeFromVdCl",
]

def calculateConstant(pkConstant: float) -> float:
    """Returns the half-life or constant (interchangeable).

    :param pkConstant: constant or half-life
    :return: constant or half-life"""
    return math.log(2) / pkConstant

def halflifeFromVdCl(vd: float, cl: float) -> float:
    """Calculates the half-life with volume of distribution and clearance rate.

    :param vd: volume of distribution
    :param cl: clearance rate"""
    return math.log(2) * vd / cl



"""Python tools for pharmacokinetic calculations."""

from . import single_dose
from . import multiple_doses
from .shared_formulas import *

__version__ = "0.1"
__author__ = "xyzpw"
__description__ = "Python tools for pharmacokinetic calculations."
__license__ = "MIT"



import math
from pharmacokinetics import _handlers

__all__ = [
    "calculateConcentration",
    "calculateRemaining",
]

def calculateConcentration(dose: float, vd: float, bioavailability: float = 1) -> float:
    """Calculates the concentration of a substance.

    :param dose: the dosage of the substance administered
    :param vd: volume of distribution
    :param bioavailability: fraction of the drug absorbed into the systemic circulation (default 1)"""
    return (dose * bioavailability) / vd

def calculateRemaining(concentration: float, elapsed: float, **kwargs) -> float:
    """Calculates the concentration remaining after a specified time.

    :param concentration: the initial concentration
    :param elapsed: time passed since peak concentrations were reached
    :param **kwargs:
        ke: elimination rate constant
        t12: elimination half-life (alternative to `ke`)"""
    ke = _handlers._getPkConstantFromKwargs("ke", "t12", kwargs)
    return concentration * math.e**(-ke * elapsed)



import math
from pharmacokinetics import _handlers

__all__ = [
    "calculateTmax",
]

def calculateTmax(**kwargs) -> float:
    """Calculates the time to reach maximum concentration (tmax).

    :param **kwargs:
        interval: the interval between dose administration (optional)
        ka: absorption rate constant
        t12abs: absorption half-life (alternative to `ka`)
        ke: elimination rate constant
        t12: elimination half-life (alternative to `ke`)"""
    ka = _handlers._getPkConstantFromKwargs("ka", "t12abs", kwargs)
    ke = _handlers._getPkConstantFromKwargs("ke", "t12", kwargs)
    interval = kwargs.get("interval")
    isMultipleDoses = interval != None

    if isMultipleDoses:
        numeratorComponents = []
        numeratorComponents.append(ka * (1 - math.exp(-ke * interval)))
        numeratorComponents.append(ke * (1 - math.exp(-ka * interval)))
        numerator = math.log(numeratorComponents[0] / numeratorComponents[1])
        return numerator / (ka - ke)
    if ka == ke:
        return 1 / ka
    return 1 / (ka - ke) * math.log(ka / ke)



import math

__all__ = [
    "halflifeFromDoses",
]

def halflifeFromDoses(dose, remaining, elapsed) -> float:
    """Calculates the half-life based on the dose remaining after a specific time.

    :param dose: the initial dose administered
    :param remaining: the remaining dose at the specified time
    :param elapsed: the time elapsed since the drug has reached peak concentrations"""
    halfLivesPassed = math.log10(remaining/dose) / math.log10(0.5)
    return elapsed / halfLivesPassed



import math
from pharmacokinetics import _handlers

__all__ = [
    "calculateCmax",
]

def calculateCmax(concentration: float, interval: float, **kwargs) -> float:
    """Calculates the maximum concentration (cmax).

    :param concentration: initial administered concentration
    :param interval: interval between administered doses
    :param **kwargs:
        ke: elimination rate constant
        t12: elimination half-life (alternative to `ke`)"""
    ke = _handlers._getPkConstantFromKwargs("ke", "t12", kwargs)
    return concentration / (1 - math.e**(-ke * interval))



import math

__all__ = [
    "_getPkConstantFromKwargs",
]

def _getPkConstantFromKwargs(constValueName: str, t12ValueName: str, kwargs: dict):
    """Returns the value of `constValueName` if it exists within the `kwargs` dictionary, otherwise it returns `t12ValueName`."""
    return kwargs.get(constValueName) if kwargs.get(constValueName) != None else math.log(2) / kwargs.get(t12ValueName)



import math
from pharmacokinetics import _handlers

__all__ = [
    "calculateRecommendedInterval",
]

def calculateRecommendedInterval(cmaxDesired, cminDesired, **kwargs) -> float:
    """Calculates the recommended interval between doses.

    :param cmaxDesired: the desired maximum concentration
    :param cminDesired: the desired minimum concentration
    :param **kwargs:
        ke: elimination rate constant
        t12: elimination half-life (alternative to `ke`)
        infusion_duration: duration of infusion (default 0)"""
    ke = _handlers._getPkConstantFromKwargs("ke", "t12", kwargs)
    infusion_duration = kwargs.get("infusion_duration", 0)
    return math.log(cmaxDesired/cminDesired) / ke + infusion_duration
